<?php
// $Id$

/**
 * Page to administer shortcuts sets
 *
 * Presents the admin user an overview page with all the shortcuts sets in the system
 *
 * @todo Use draggable tables to control the shortcuts order of appearance
 *
 * @return
 * A table with all shorcuts sets in the system
 */
function toolbar_shortcuts_set_admin() {
  $result = db_query('SELECT * FROM {toolbar_shortcuts_set}');
  $header = array(
    array('data' => t('Name')),
    array('data' => t('Operations'), 'colspan' => '3')
  );

  $default_set = variable_get('toolbar_shortcuts_default_set', null);

  $rows = array();
  $destination = drupal_get_destination();
  while ($data = db_fetch_object($result)) {
    $is_default = ($default_set == $data->set_name);

    $row = array(
      check_plain($data->title) . ( ($is_default) ? '&nbsp;(' . t('default') . ')' : '' ),
      l(t('list links'), "admin/build/shortcuts/$data->set_name"),
      l(t('edit set name'), "admin/build/shortcuts/$data->set_name/edit", array('query' => $destination)),
      l(t('export'), "admin/build/shortcuts/$data->set_name/export", array('query' => $destination)),
      ($is_default ? '' : l(t('delete'), "admin/build/shortcuts/$data->set_name/delete", array('query' => $destination)))
    );
    $rows[] = $row;
  }

  if (empty($rows)) {
    $empty_message = t('No shortcut sets found. !create', array('!create' => l(t('Create a new one.'), 'admin/build/shortcuts/add-set')));
    $rows[] = array(array('data' => $empty_message, 'colspan' => '4'));
  }

  $output .= theme('table', $header, $rows);

  return $output;
}

/**
 * Form to add and edit shortcuts sets
 *
 * This form is used to add and edit shortcuts sets deppending on the context
 *
 * @param $edit
 * An array with the shortcut set 
 *
 * @return
 * A form to add or edit a shortcut set
 */
function toolbar_shortcuts_set_form(&$form_state, $set = array('title' => '', 'set_name' => '')) {

  if ( empty($set['set_name']) ) {
    $form['set_name'] = array(
      '#type' => 'textfield',
      '#title' => t('Name'),
      '#default_value' => $set['set_name'],
      '#maxsize' => TOOLBAR_MAX_SET_NAME_LENGTH_UI,
      '#description' => t('The machine-readable name of this set. This name must contain only lowercase letters, numbers, and hyphens, and must be unique.'),
      '#required' => TRUE,
    );
    $form['#insert'] = TRUE;
  }
  else {
    $form['set_name'] = array('#type' => 'value', '#value' => $set['set_name']);
  }

  $form['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Title'),
    '#default_value' => $set['title'],
    '#maxlength' => 128,
    '#size' => 45,
    '#description' => t('Specify title for this shortcut set.'),
    '#required' => TRUE,
  );

  $form['submit'] = array('#type' => 'submit', '#value' => t('Save'));
  return $form;
}

/**
 * Validation handler for the toolbar_shortcuts_set_form
 *
 * If adding new set we must check for unique set name.
 *
 * @see toolbar_shortcuts_set_form()
 */
function toolbar_shortcuts_set_form_validate($form, &$form_state) {
  $set = $form_state['values'];
  if (preg_match('/[^a-z0-9-]/', $set['set_name'])) {
    form_set_error('set_name', t('The set name may only consist of lowercase letters, numbers, and hyphens.'));
  }
  if (strlen($set['set_name']) > TOOLBAR_MAX_SET_NAME_LENGTH_UI) {
    form_set_error('set_name', format_plural(TOOLBAR_MAX_SET_NAME_LENGTH_UI, "The set name can't be longer than 1 character.", "The set name can't be longer than @count characters."));
  }
  if ($form['#insert']) {
    // We will add 'set-' to the set name to help avoid name-space conflicts.
    $set['set_name'] = 'set-'. $set['set_name'];
    if (db_result(db_query("SELECT set_name FROM {toolbar_shortcuts_set} WHERE set_name = '%s'", $set['set_name'])) ||
      db_result(db_query_range("SELECT menu_name FROM {menu_links} WHERE menu_name = '%s'", $set['set_name'], 0, 1))) {
      form_set_error('set_name', t('The set already exists.'));
    }
  }
}

/**
 * Submit handler for the toolbar_shortcuts_set_form
 *
 * @see toolbar_shortcuts_set_form()
 *
 * @return
 * A success message if the shortcut et was saved
 */
function toolbar_shortcuts_set_form_submit($form, &$form_state) {
  $set = $form_state['values'];

  if ($form['#insert']) {
    // Add 'set-' to the set name to help avoid name-space conflicts.
    $set['set_name'] = 'set-'. $set['set_name'];
    db_query("INSERT INTO {toolbar_shortcuts_set} (title, set_name) VALUES ('%s', '%s')", $set['title'], $set['set_name']);
    drupal_set_message(t('The %title shortcut set has been created.', array('%title' => $set['title'])));
  }
  else {
    db_query("UPDATE {toolbar_shortcuts_set} SET title = '%s' WHERE set_name = '%s'", $set['title'], $set['set_name']);
    drupal_set_message(t('The %title shortcut set has been updated.', array('%title' => $set['title'])));
  }
  $form_state['redirect'] = 'admin/build/shortcuts';
}

/**
 * A form to edit a shortcut set
 *
 * @param $set_name
 *  The shortcut set name
 *
 * @see toolbar_shortcuts_set_load()
 * @see toolbar_shortcuts_set_form()
 *
 * @return
 *  An empty form if we're creating a new shortcut set or
 *  a form with the shortcut set values where editing
 */
function toolbar_shortcuts_set_edit($set_name) {
  if ($set_name) {
    $set = toolbar_shortcuts_set_load($set_name);
    drupal_set_title(t('Edit shortcut set "!shortcut"', array('!shortcut' => check_plain($set['title']))));
    $output = drupal_get_form('toolbar_shortcuts_set_form', $set);
  }
  else {
    $output = drupal_get_form('toolbar_shortcuts_set_form');
  }

  return $output;
}

/**
 * Confirmation page to delete a shortcut set
 *
 * We use this function to present a confirmation page to delete a shortcut set and all links
 * inside that set
 *
 * @param $set_name
 * the shortcut set name
 * we pass the set name of the shortcut we want to delete
 *
 * @see toolbar_shortcuts_set_load()
 * @see toolbar_shortcuts_set_delete()
 * @see toolbar_shortcuts_set_delete_confirm_submit
 *
 * @return
 * a confirmation page to delete a shortcut set
 */
function toolbar_shortcuts_set_delete_confirm($form_state, $set_name) {
  $set = toolbar_shortcuts_set_load($set_name);
  if (user_access('administer shortcuts')) {
    $form['set_name'] = array('#type' => 'value', '#value' => $set_name);
    $output = confirm_form($form,
      t('Are you sure you want to delete the shortcut set %title? All links inside the set will be deleted too and all users who are using this set will be asigned with the default set.', array('%title' => check_plain($set['title']))),
      isset($_GET['destination']) ? $_GET['destination'] : 'admin/build/shortcuts');
  }
  return $output;
}

/**
 * Submit handler to delete a shortcut set
 *
 * Passes the $set_name from the form in toolbar_shortcuts_set_delete_confirm() to
 * toolbar_shortcuts_set_delete() so we can delete the shortcut set
 *
 * @see toolbar_shortcuts_set_delete()
 * @see toolbar_shortcuts_set_delete_confirm()
 *
 * @ingroup forms
 */
function toolbar_shortcuts_set_delete_confirm_submit($form, &$form_state) {
  if ($form_state['values']['confirm']) {
    toolbar_shortcuts_set_delete($form_state['values']['set_name']);
    $form_state['redirect'] = 'admin/build/shortcuts';
    return;
  }
}

/**
 * Deletes a shortcut set based on the set name and all links related to that set
 *
 * @param $set_name
 *    The the machine-readable name of the set.
 *
 * @see toolbar_shortcuts_set_delete_confirm_submit()
 */
function toolbar_shortcuts_set_delete($set_name) {
  // check if this is the default set
  $default_set = variable_get("toolbar_shortcuts_default_set", '');
  if ( $set_name == $default_set ) {
    drupal_set_message(t('The default shortcut set can\'t be been deleted.'));
  } else {
    // First, assign all users using this set to the default set
    $users = db_query("SELECT uid FROM {toolbar_shortcuts_users} WHERE set_name = '%s'", $set_name);
    while($uid = db_fetch_object($users)) {
      $account = user_load($uid);
      toolbar_shortcuts_set_assign_user($default_set, $account);
    }

    // look for all links with this set name
    $result = db_query("SELECT mlid FROM {menu_links} WHERE menu_name = '%s'", $set_name);
    while($link = db_fetch_object($result)) {
      // delete all links with this set name
      drupal_set_message(t('The shortcut !link_title has been deleted.', array('!link_title' => $link->link_title)));
      menu_link_delete($link->mlid);
    }
    // finally, delete the set
    db_query("DELETE FROM {toolbar_shortcuts_set} WHERE set_name = '%s'", $set_name);

    drupal_set_message(t('The shortcut set has been deleted.'));
  }
}


/**
 * Toolbart Shortcuts export form.
 *
 *
 * @param $set_name
 *  The machine-readable name of the set to export
 *
 * @return
 *  Form array
 *
 * @ingroup forms
 */
function toolbar_shortcuts_export_form(&$form_state, $set_name) {

  $form = array();
  $set = array();

  // get all set links
  $result = db_query("SELECT mlid FROM {menu_links} WHERE menu_name = '%s'", $set_name);
  while($link = db_fetch_object($result)) {
    $link = menu_link_load($link->mlid);
    $set[] = array(
      'link_path' => $link['link_path'],
      'link_title' => $link['link_title'],
      'weight' => $link['weight'],
    );
  }
  $data = '$toolbar = ' . var_export($set, TRUE) . ';';
  $lines = substr_count($data, "\n") + 1;

  $form['data'] = array(
    '#type' => 'textarea',
    '#title' => t('Data'),
    '#description' => 'Copy this value.',
    '#value' => $data,
    '#rows' => min($lines, 150),
  );

  return $form;
}


/**
 * Toolbart Shortcuts import form.
 *
 * @return
 *  Form array
 *
 * @see toolbar_shortcuts_import_form_validate()
 * @see toolbar_shortcuts_import_form_submit()
 * @ingroup forms
 */
function toolbar_shortcuts_import_form(&$form_state) {
  $form = array();

  $form['#insert'] = TRUE;
  $form['#validate'][] = 'toolbar_shortcuts_set_form_validate';
  $form['#validate'][] = 'toolbar_shortcuts_import_form_validate';

  $form['set_name'] = array(
    '#type' => 'textfield',
    '#title' => t('Name'),
    '#maxsize' => TOOLBAR_MAX_SET_NAME_LENGTH_UI,
    '#description' => t('The machine-readable name of this set. This name must contain only lowercase letters, numbers, and hyphens, and must be unique.'),
    '#required' => TRUE,
  );

  $form['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Title'),
    '#description' => t('Specify title for this shortcut set.'),
    '#required' => TRUE,
  );

  $form['data'] = array(
    '#type' => 'textarea',
    '#title' => t('Toolbar export data'),
    '#description' => t('Paste set data as copied from export.'),
    '#rows' => 16,
    '#required' => TRUE,
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Import'),
  );

  return $form;
}

/**
 * Validation handler for Toolbar Shortcuts import form.
 *
 * @see toolbar_shortcuts_import_form()
 * @see toolbar_shortcuts_import_form_submit()
 *
 * @ingroup forms
 */
function toolbar_shortcuts_import_form_validate(&$form, &$form_state) {
  ob_start();
  eval($form_state['values']['data']);
  ob_end_clean();

  if ( empty($toolbar) ) {
    form_error($form['data'], t('Pasted data is not valid Set object.'));
  }
  else {
    foreach ($toolbar as $link) {
      if (
        !isset($link['link_path']) || empty($link['link_path']) ||
        !isset($link['link_title']) || empty($link['link_title']) ||
        !isset($link['weight']) || '' == $link['weight']
      ) {
        form_error($form['data'], t('Pasted data is not valid Shortcut set export.'));
        break;
      }
    }
  }
  $form_state['values']['shortcuts'] = $toolbar;
}

/**
 * Submit handler for Toolbar shortcuts import form.
 *
 * Uses toolbar_shortcuts_set_form_submit() to create new set. Adds the minimal 
 * requirement for new link and creates shortcuts with menu_link_save().
 *
 * @see toolbar_shortcuts_set_form_submit()
 * @see menu_link_save()
 * @see toolbar_shortcuts_import_form()
 * @see toolbar_shortcuts_import_form_validate()
 *
 * @ingroup forms
 */
function toolbar_shortcuts_import_form_submit(&$form, &$form_state) {
  foreach($form_state['values']['shortcuts'] as $link) {
    $link['module'] = 'toolbar';
    $link['menu_name'] = 'set-' . $form_state['values']['set_name'];
    menu_link_save($link);
  }
  toolbar_shortcuts_set_form_submit(&$form, &$form_state);
  drupal_set_message(t('The %title shortcut set has been imported.', array('%title' => $set['title'])));
}
