<?php
// $Id$

/**
 * @file
 * Toolbar Shortucts administration handlers
 *
 * Functions and form to create, edit and delete shortcuts. Functions and forms
 * to create, edit and delete set of shortcuts.
 */


/**
 * Page to administer shortcuts
 *
 * Presents the admin user an overview page with all the shortcuts sets in the system
 *
 * @param $set_name
 *  The machine readable name of the set to edit
 *
 * @return
 *  A form array with all shorcuts for given set
 *
 * @see toolbar_shortcuts_admin_form_submit()
 * @see theme_toolbar_shortcuts_admin_form()
 * @ingroup forms
 */
function toolbar_shortcuts_links_admin_form(&$form_state, $set_name = NULL) {
  $set = toolbar_shortcuts_set_load($set_name);
  $result = db_query("SELECT mlid, link_title, link_path, weight FROM {menu_links} WHERE menu_name = '%s' ORDER BY weight ASC", $set['set_name']);

  $destination = drupal_get_destination();
  $delta = count($data);

  $form = array();
  $form['weight'] = array('#tree' => TRUE);

  while ($data = db_fetch_object($result)) {

    $form['weight'][$data->mlid] = array(
      '#type' => 'weight',
      '#default_value' => $data->weight,
      '#attributes' => array('class' => 'toolbar-links-order-weight')
    );
    $form['id'][$data->mlid] = array('#type' => 'hidden', '#value' => $data->mlid);

    $form['title'][$data->mlid] = array('#value' => l(t($data->link_title), check_plain($data->link_path)));

    $form['operations'][$data->mlid] = array(
      '#value' =>
        // edit link
        l(t('edit'), "admin/build/shortcuts/edit/{$data->mlid}", array('query' => $destination))
        . '&nbsp;' .
        // delete link
        l(t('delete'), "admin/build/shortcuts/{$set['setid']}/delete/{$data->mlid}", array('query' => $destination))
    );
  }

  $form['set_name'] = array('#type' => 'hidden', '#value' => $set['set_name']);
  $form['submit'] = array('#type' => 'submit', '#value' => t('Save configuration'));

  return $form;
}

/**
 * Submit handler for toolbar_shortcuts_links_admin_form()
 *
 *
 * @see toolbar_shortcuts_links_admin_form()
 */
function toolbar_shortcuts_links_admin_form_submit($form, &$form_state) {
  foreach ($form_state['values']['weight'] as $mlid => $weight) {
    // update only modified items
    if ( $form['weight'][$mlid]['#default_value'] != $weight ) {
      $link = menu_link_load($mlid);
      $link['weight'] = $weight;
      menu_link_save($link);
    }
  }
  drupal_set_message(t('The shortcut ordering has been saved.'));
}

/**
 * Theme handler for toolbar_shortcuts_links_admin_form()
 *
 * @see theme_toolbar_shortcuts_admin_form()
 * @ingroup themable
 */
function theme_toolbar_shortcuts_links_admin_form($form) {

  $rows = array();
  foreach(element_children($form['title']) as $mlid) {
    if ( is_array( $form['title'][$mlid] ) ) {
      $rows[] = array(
        'data' => array(
          drupal_render($form['title'][$mlid]),
          drupal_render($form['operations'][$mlid]),
          drupal_render($form['weight'][$mlid]) . drupal_render($form['id'][$mlid]),
        ),
        'class' => 'draggable'
      );
    }
  }

  if (empty($rows)) {
    $empty_message = t("There are no shortcut items yet. !create", array("!create" => l(t("Create a new one."), "admin/build/shortcuts/{$form['set_name']['#value']}/add")));
    $rows[] = array(array('data' => $empty_message, 'colspan' => '2'));
  }

  $header = array(t('Title'), array('data' => t('Operations'), 'colspan' => 2) );
  $output = theme('table', $header, $rows, array('id' => 'toolbar-links-order'));
  $output .= drupal_render($form);

  drupal_add_tabledrag('toolbar-links-order', 'order', 'sibling', 'toolbar-links-order-weight');

  return $output;
}

function toolbar_shortcuts_link_add_form(&$form_state, $set_name, $edit = array('mlid' => NULL)) {
  $form['#shortcut'] = $edit;

  $form['mlid'] = array(
    '#type' => 'hidden',
    '#default_value' => $edit['mlid'] ? $edit['mlid'] : 0,
  );

  $form['link_title'] = array(
    '#type' => 'textfield',
    '#title' => t('Shortcut name'),
    '#default_value' => $edit['link_title'],
    '#maxlength' => 128,
    '#size' => 45,
    '#description' => t('Specify a name for this shortcut.'),
    '#required' => TRUE,
  );
  $form['link_path'] = array(
    '#type' => 'textfield',
    '#title' => t('Path'),
    '#default_value' => $edit['link_path'],
    '#maxlength' => 128,
    '#size' => 45,
    '#description' => t('Specify a path by which this shortcut can be accessed.'),
    '#required' => TRUE,
  );
  
  $form['set_name'] = array(
    '#type' => 'hidden',
    '#default_value' => $edit['set_name'] ? $edit['set_name'] : $set_name,
  );
  $form['weight'] = array(
    '#type' => 'hidden',
    '#default_value' => isset($edit['weight']) ? $edit['weight'] : 0,
  );
  if ($edit['mlid']) {
    $form['mlid'] = array('#type' => 'hidden', '#value' => $edit['mlid']);
    $form['submit'] = array('#type' => 'submit', '#value' => t('Update shortcut'));
  }
  else {
    $form['submit'] = array('#type' => 'submit', '#value' => t('Create new shortcut'));
  }

  return $form;
}


/**
 * Submit handler for the toolbar_shortcuts_link_add_form
 *
 * Submits the form, build the $link object and then save the link using
 * menu_link_save() to insert the shortcut as a link in the menu system
 *
 * @see toolbar_shortcuts_link_add_form()
 * @see toolbar_shortcuts_set_load()
 * @see menu_link_save()
 *
 * @return
 * A success message if the shortcut was saved
 */
function toolbar_shortcuts_link_add_form_submit($form, &$form_state) {
  $set = toolbar_shortcuts_set_load($form_state['values']['set_name']);
  
  $link = array(
    'menu_name' => $set['set_name'],
    'link_path' => $form_state['values']['link_path'],
    'plid' => 0,
    'mlid' => $form_state['values']['mlid'],
    'link_title' => $form_state['values']['link_title'],
    'weight' => $form_state['values']['weight'],
    'module' => 'toolbar',
  );
  
  menu_link_save($link);
  
  drupal_set_message(t('The %link shortcut has been created.', array('%link' => $form_state['values']['link_title'])));
  $form_state['redirect'] = 'admin/build/shortcuts/'.$form_state['values']['set_name'];
  return;
}


/** 
 * A form to edit a shortcut set
 *
 * If we pass a valid $setid then we use toolbar_shortcuts_load() to load an object in the $shortcut variable
 * then we use the toolbar_shortcuts_admin_form() to edit the shortcut.
 *
 * If we pass no valid $sid then the shortcut doesn't exists and we need to create it
 *
 * @param $sid
 * The shortcut unique id
 *
 * @see toolbar_shortcuts_load()
 * @see toolbar_shortcuts_admin_form()
 *
 * @return
 * an empty form if we're creating a new shortcut or
 * a form with the shortcut values
 */
function toolbar_shortcuts_link_edit($mlid) { 
  if ($mlid) {
    $link = menu_link_load($mlid);
    drupal_set_title(t('Edit shortcut "!shortcut"', array('!shortcut' => check_plain($link['link_title']))));
    $output = drupal_get_form('toolbar_shortcuts_link_add_form', $link['menu_name'], $link);
  }
  else {
    $output = drupal_get_form('toolbar_shortcuts_link_add_form');
  } 
  
  return $output;
}

/**
 * Confirmation page to delete a shortcut
 *
 * We use this function to present a confirmation page to delete a shortcut
 *
 * @param $sid
 * the shortcut unique id
 * we pass the $sid of the shortcut we want to delete
 *
 * @see toolbar_shortcuts_overview()
 *
 * @return
 * a confirmation page to delete a shortcut
 */
function toolbar_shortcuts_link_delete_confirm($form_state, $set_name, $mlid) {
  $link = menu_link_load($mlid);
  if (user_access('administer shortcuts')) {
    $form['mlid'] = array('#type' => 'value', '#value' => $mlid);
    $output = confirm_form($form,
      t('Are you sure you want to delete the shortcut %title?', array('%title' => check_plain($link['link_title']))),
      isset($_GET['destination']) ? $_GET['destination'] : "admin/build/shortcuts/{$set['set_name']}");
  }
  return $output;
}

/**
 * Submit handler to delete a shortcut
 *
 * Passes the $sid from the form in toolbar_shortcuts_delete_confirm() to
 * toolbar_shortcuts_admin_delete() so we can delete the shortcut
 *
 * @see toolbar_shortcuts_admin_delete_confirm()
 */
function toolbar_shortcuts_link_delete_confirm_submit($form, &$form_state) {
  if ($form_state['values']['confirm']) {
    toolbar_shortcuts_link_delete($form_state['values']['mlid']);
    $form_state['redirect'] = "admin/build/shortcuts/{$set['set_name']}";
    return;
  }
}

/**
 * Deletes a shortcut based on the shortcut unique id
 *
 * @param $sid
 * The $sid of the shortcut we want to delete
 *
 * @see toolbar_shortcuts_admin_delete_confirm_submit()
 */
function toolbar_shortcuts_link_delete($mlid = 0) {
  menu_link_delete($mlid);
  drupal_set_message(t('The shortcut has been deleted.'));
}
